/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 

*/
 
 
package cnrs.i3s.papareto;

import java.util.Random;

public abstract class MutationOperator<R> extends Operator
{
	private double occurenceProbability = 0.5;
	private double power = 1;

	public void mutate(R e, Random r)
	{
		for (int i = 0; i < numberOfChanges; ++i)
		{
			performOneSingleChange(e, r);
		}
	}

	protected abstract void performOneSingleChange(R individual, Random prng);

	public double getProbability()
	{
		return occurenceProbability;
	}

	public void setProbability(double occurenceProbability)
	{
		this.occurenceProbability = occurenceProbability;
	}

	public int getNumberOfChanges()
	{
		return numberOfChanges;
	}

	public void setNumberOfChanges(int numberOfChanges)
	{
		if (numberOfChanges < 0)
			throw new IllegalArgumentException("numberOfOccurences must be >= 0");

		this.numberOfChanges = numberOfChanges;
	}

}
