/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 

*/
 
 
package cnrs.i3s.papareto;

import java.io.Serializable;
import java.util.List;

public class Operator implements Serializable
{
	public int nbSuccess = 0, nbFailure = 0;

	public double getSuccessRate()
	{
		return nbSuccess / (double) (nbSuccess + nbFailure);
	}

	@Override
	public String toString()
	{
		return getClass().getName() + " succes rate=" + getSuccessRate();
	}

	public String getFriendlyName()
	{
		return getClass().getName();
	}
	
	public static <O extends Operator> double[] getUsageProbabilities(List<O> operators)
	{
		double[] a = new double[operators.size()];

		for (int i = 0; i < a.length; ++i)
		{
			Operator o = operators.get(i);

			// if the operator was never used in the past
			if (o.nbFailure + o.nbSuccess == 0)
			{
				// give it max opportunity by assuming it would have top performance
				a[i] = 1;
			}
			else
			{
				a[i] = operators.get(i).getSuccessRate();
			}
		}

		return a;
	}
}
