/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 

*/
 
 
package cnrs.i3s.papareto.demo.tsp;

import java.util.Arrays;
import java.util.List;
import java.util.Random;

import toools.text.CSV;

public class Map
{
	final int nbCities;
	final int[][] distances;

	public Map(int nbCities)
	{
		this.nbCities = nbCities;
		distances = new int[nbCities][];

		for (int i = 0; i < nbCities; ++i)
		{
			distances[i] = new int[nbCities];
			Arrays.fill(distances[i], - 1);
		}
	}

	public void setDistance(int c1, int c2, int d)
	{
		distances[c1][c2] = d;
		distances[c2][c1] = d;
	}

	public int getDistance(int c1, int c2)
	{
		return distances[c1][c2];
	}

	public static Map load(String s)
	{
		List<List<String>> lines = CSV.disassemble(s, "\t");

		Map m = new Map(Integer.valueOf(lines.get(0).get(0)));

		for (int i = 1; i < lines.size(); ++i)
		{
			List<String> line = lines.get(i);
			int c1 = Integer.valueOf(line.get(0));
			int c2 = Integer.valueOf(line.get(1));
			int d = Integer.valueOf(line.get(2));
			m.setDistance(c1, c2, d);
		}

		return m;
	}

	public void randomize()
	{
		Random r = new Random();

		for (int c = 0; c < nbCities; ++c)
		{
			for (int d = c + 1; d < nbCities; ++d)
			{
				setDistance(c, d, r.nextInt(100));
			}
		}
	}
}
