/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 /////////////////////////////////////////////////////////////////////////////////////////
// 
//                 Université de Nice Sophia-Antipolis  (UNS) - 
//                 Centre National de la Recherche Scientifique (CNRS)
//                 Copyright © 2015 UNS, CNRS All Rights Reserved.
// 
//     These computer program listings and specifications, herein, are
//     the property of Université de Nice Sophia-Antipolis and CNRS
//     shall not be reproduced or copied or used in whole or in part as
//     the basis for manufacture or sale of items without written permission.
//     For a license agreement, please contact:
//     <mailto: licensing@sattse.com> 
//
//
//
//     Author: Luc Hogie – Laboratoire I3S - luc.hogie@unice.fr
//
//////////////////////////////////////////////////////////////////////////////////////////

package dht.set;

import java.util.Arrays;
import java.util.Iterator;
import java.util.Random;

import com.carrotsearch.hppc.IntArrayList;
import com.carrotsearch.hppc.cursors.IntCursor;

import dht.hppcutils.HPPCUtils;
import toools.collection.bigstuff.longset.LongCursor;

public class ArrayListLongSet32 extends LongSet32
{
	private final IntArrayList hppcSet;

	public ArrayListLongSet32()
	{
		this.hppcSet = new IntArrayList();
	}

	public ArrayListLongSet32(int capacity)
	{
		this.hppcSet = new IntArrayList(capacity);
	}

	@Override
	public Class<?> getImplementationClass()
	{
		return IntArrayList.class;
	}

	@Override
	public void add(long e)
	{		assert e < Integer.MAX_VALUE;

		hppcSet.add((int) e);
		// fireAdd(e);
	}

	public void sort()
	{
		Arrays.sort(hppcSet.buffer, 0, hppcSet.elementsCount);
	}

	@Override
	public boolean removeFirstOccurence(long e)
	{		assert e < Integer.MAX_VALUE;

		return hppcSet.removeFirstOccurrence((int) e) >= 0;
	}

	@Override
	public int removeAllOccurences(long e)
	{
		assert e < Integer.MAX_VALUE;
		return hppcSet.removeAllOccurrences((int) e);
	}

	@Override
	public long pickRandomElement(Random prng)
	{
		return HPPCUtils.pickRandomElement(hppcSet, prng);
	}

	@Override
	public void clear()
	{
		hppcSet.clear();
	}

	@Override
	public boolean contains(long e)
	{		assert e < Integer.MAX_VALUE;

		return hppcSet.contains((int) e);
	}

	@Override
	public int size()
	{
		return hppcSet.size();
	}

	@Override
	public boolean isEmpty()
	{
		return hppcSet.isEmpty();
	}

	@Override
	public Iterator<LongCursor> iterator()
	{
		Iterator<IntCursor> i = hppcSet.iterator();

		return new Iterator<LongCursor>()
		{
			final LongCursor longCursor = new LongCursor();

			@Override
			public boolean hasNext()
			{
				return i.hasNext();
			}

			@Override
			public LongCursor next()
			{
				// TODO Auto-generated method stub
				IntCursor ic = i.next();
				longCursor.value = ic.value;
				return longCursor;
			}

		};
	}

	public long[] toLongArray()
	{
		long[] r = new long[size()];
		System.arraycopy(hppcSet, 0, r, 0, hppcSet.elementsCount);
		return r;
	}

	public int[] toIntArray()
	{
		int[] r = new int[size()];
		System.arraycopy(hppcSet, 0, r, 0, hppcSet.elementsCount);
		return r;
	}

	@Override
	public long getMemoryFootprintInBytes()
	{
		return hppcSet.buffer.length * 4;
	}
}
