/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 /////////////////////////////////////////////////////////////////////////////////////////
// 
//                 Université de Nice Sophia-Antipolis  (UNS) - 
//                 Centre National de la Recherche Scientifique (CNRS)
//                 Copyright © 2015 UNS, CNRS All Rights Reserved.
// 
//     These computer program listings and specifications, herein, are
//     the property of Université de Nice Sophia-Antipolis and CNRS
//     shall not be reproduced or copied or used in whole or in part as
//     the basis for manufacture or sale of items without written permission.
//     For a license agreement, please contact:
//     <mailto: licensing@sattse.com> 
//
//
//
//     Author: Luc Hogie – Laboratoire I3S - luc.hogie@unice.fr
//
//////////////////////////////////////////////////////////////////////////////////////////

package dht.set;

import java.util.Arrays;
import java.util.Iterator;
import java.util.Random;

import com.carrotsearch.hppc.LongArrayList;

import dht.hppcutils.HPPCUtils;
import toools.collection.bigstuff.longset.LongCursor;

public class ArrayListLongSet64 extends LongSet64
{
	private final LongArrayList hppcSet;

	public ArrayListLongSet64()
	{
		this.hppcSet = new LongArrayList();
	}

	public ArrayListLongSet64(int capacity)
	{
		this.hppcSet = new LongArrayList(capacity);
	}

	@Override
	public Class<?> getImplementationClass()
	{
		return LongArrayList.class;
	}

	@Override
	public void add(long e)
	{
		hppcSet.add(e);
		// fireAdd(e);
	}

	public void sort()
	{
		Arrays.sort(hppcSet.buffer, 0, hppcSet.elementsCount);
	}

	@Override
	public boolean removeFirstOccurence(long e)
	{
		return hppcSet.removeFirstOccurrence(e) >= 0;
	}

	@Override
	public int removeAllOccurences(long e)
	{
		return hppcSet.removeAllOccurrences(e);
	}

	@Override
	public long pickRandomElement(Random prng)
	{
		return HPPCUtils.pickRandomElement(hppcSet, prng);
	}

	@Override
	public void clear()
	{
		hppcSet.clear();
	}

	@Override
	public boolean contains(long id)
	{
		return hppcSet.contains(id);
	}

	@Override
	public int size()
	{
		return hppcSet.size();
	}

	@Override
	public boolean isEmpty()
	{
		return hppcSet.isEmpty();
	}

	@Override
	public Iterator<LongCursor> iterator()
	{
		return LongCursor.fromHPPC(hppcSet).iterator();
	}

	public long[] toArray()
	{
		long[] r = new long[size()];
		System.arraycopy(hppcSet, 0, r, 0, hppcSet.elementsCount);
		return r;
	}

	@Override
	public long getMemoryFootprintInBytes()
	{
		return hppcSet.buffer.length * 4;
	}
}
