/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 /////////////////////////////////////////////////////////////////////////////////////////
// 
//                 Université de Nice Sophia-Antipolis  (UNS) - 
//                 Centre National de la Recherche Scientifique (CNRS)
//                 Copyright © 2015 UNS, CNRS All Rights Reserved.
// 
//     These computer program listings and specifications, herein, are
//     the property of Université de Nice Sophia-Antipolis and CNRS
//     shall not be reproduced or copied or used in whole or in part as
//     the basis for manufacture or sale of items without written permission.
//     For a license agreement, please contact:
//     <mailto: licensing@sattse.com> 
//
//
//
//     Author: Luc Hogie – Laboratoire I3S - luc.hogie@unice.fr
//
//////////////////////////////////////////////////////////////////////////////////////////


package dht.set;

import java.util.Iterator;
import java.util.Random;

import toools.collection.bigstuff.longset.LongCursor;


public class ArrayLongSet32 extends LongSet32
{
	public final int[] array;

	public ArrayLongSet32(int[] array)
	{
		this.array = array;
	}

	public ArrayLongSet32(long[] longArray)
	{
		this.array = new int[longArray.length];
		int sz = longArray.length;
		long l;

		for (int i = 0; i < sz; ++i)
		{
			l = longArray[i];
			assert l <= Integer.MAX_VALUE;
			this.array[i] = (int) l;
		}
	}

	@Override
	public Class<?> getImplementationClass()
	{
		return int[].class;
	}

	@Override
	public void add(final long n)
	{
		throw new IllegalStateException();
	}

	@Override
	public boolean removeFirstOccurence(long n)
	{
		throw new IllegalStateException("can't remove an element from an array");
	}

	@Override
	public int removeAllOccurences(long n)
	{
		throw new IllegalStateException("can't remove an element from an array");
	}
	@Override
	public long pickRandomElement(Random prng)
	{
		return array[prng.nextInt(array.length)];
	}

	@Override
	public void clear()
	{
		throw new IllegalStateException();
	}

	@Override
	public boolean contains(long n)
	{
		if (array == null)
		{
			return false;
		}
		else
		{
			return indexOf(n) >= 0;
		}
	}

	public int indexOf(long n)
	{
		for (int i = 0; i < array.length; ++i)
		{
			if (array[i] == n)
			{
				return i;
			}
		}

		return -1;
	}

	@Override
	public int size()
	{
		return array == null ? 0 : array.length;
	}

	@Override
	public boolean isEmpty()
	{
		return array == null || array.length == 0;
	}

	@Override
	public Iterator<LongCursor> iterator()
	{
		return new Iterator<LongCursor>()
		{
			int i = 0;
			final LongCursor c = new LongCursor();

			@Override
			public boolean hasNext()
			{
				return array != null && i < array.length;
			}

			@Override
			public LongCursor next()
			{
				c.value = array[i];
				return c;
			}

			@Override
			public void remove()
			{
				throw new IllegalStateException();
			}
		};
	}


	@Override
	public long getMemoryFootprintInBytes()
	{
		return array.length * 1 + 8;
	}
}
