/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 package dht.set;

import java.util.Iterator;
import java.util.Random;

import com.carrotsearch.hppc.LongOpenHashSet;
import com.carrotsearch.hppc.LongSet;

import toools.NotYetImplementedException;
import toools.collection.bigstuff.longset.LongCursor;

public class HPPCLongSetBackedLongSet extends LongSet64
{
	// HPPC objects are not serializable
	public transient LongSet hppcSet = new LongOpenHashSet();

	public HPPCLongSetBackedLongSet()
	{

	}

	public HPPCLongSetBackedLongSet(LongSet s)
	{
		this.hppcSet = s;
	}

	@Override
	public long getMemoryFootprintInBytes()
	{
		return size() * (8 + 1);
	}

	@Override
	public Class<?> getImplementationClass()
	{
		return hppcSet.getClass();
	}

	@Override
	public void add(long e)
	{
		hppcSet.add(e);
	}

	@Override
	public boolean removeFirstOccurence(long n)
	{
		return hppcSet.removeAllOccurrences(n) > 0;
	}

	@Override
	public int removeAllOccurences(long n)
	{
		return hppcSet.removeAllOccurrences(n);
	}

	@Override
	public boolean contains(long n)
	{
		return hppcSet.contains(n);
	}

	@Override
	public int size()
	{
		return hppcSet.size();
	}

	@Override
	public long pickRandomElement(Random prng)
	{
		throw new NotYetImplementedException();
	}

	@Override
	public Iterator<LongCursor> iterator()
	{
		return LongCursor.fromHPPC(hppcSet).iterator();
	}

	@Override
	public void clear()
	{
		hppcSet.clear();
	}
}
