/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 /////////////////////////////////////////////////////////////////////////////////////////
// 
//                 Université de Nice Sophia-Antipolis  (UNS) - 
//                 Centre National de la Recherche Scientifique (CNRS)
//                 Copyright © 2015 UNS, CNRS All Rights Reserved.
// 
//     These computer program listings and specifications, herein, are
//     the property of Université de Nice Sophia-Antipolis and CNRS
//     shall not be reproduced or copied or used in whole or in part as
//     the basis for manufacture or sale of items without written permission.
//     For a license agreement, please contact:
//     <mailto: licensing@sattse.com> 
//
//
//
//     Author: Luc Hogie – Laboratoire I3S - luc.hogie@unice.fr
//
//////////////////////////////////////////////////////////////////////////////////////////

package dht.set;

import java.util.Iterator;
import java.util.Random;

import com.carrotsearch.hppc.IntOpenHashSet;
import com.carrotsearch.hppc.LongOpenHashSet;
import com.carrotsearch.hppc.cursors.IntCursor;

import toools.collection.bigstuff.longset.LongCursor;
import toools.math.MathsUtilities;

public class HashLongSet32 extends LongSet32
{
	private final IntOpenHashSet hppcSet;

	public HashLongSet32()
	{
		// it is meant to store at least one element
		this(1);
	}

	public HashLongSet32(int capacity)
	{
		hppcSet = new IntOpenHashSet(capacity);
	}

	public HashLongSet32(int[] neighbors)
	{
		this(neighbors.length);
		for (int v : neighbors)
		{
			hppcSet.add(v);
		}
	}

	@Override
	public Class<?> getImplementationClass()
	{
		return LongOpenHashSet.class;
	}

	@Override
	public void add(long e)
	{
		assert e <= Integer.MAX_VALUE;
		hppcSet.add((int) e);
		// fireAdd(e);
	}

	@Override
	public boolean removeFirstOccurence(long e)
	{
		assert e <= Integer.MAX_VALUE;
		return hppcSet.remove((int) e);
	}

	@Override
	public int removeAllOccurences(long e)
	{
		assert e <= Integer.MAX_VALUE;

		return removeFirstOccurence(e) ? 1 : 0;
	}

	@Override
	public long pickRandomElement(Random prng)
	{
		while (true)
		{
			int i = MathsUtilities.pickRandomBetween(0, hppcSet.keys.length, prng);
			long n = hppcSet.keys[i];

			// if (set.contains(n))
			if (hppcSet.allocated[i])
			{
				return n;
			}
		}
	}

	@Override
	public void clear()
	{
		hppcSet.clear();
	}

	@Override
	public boolean contains(long e)
	{
		assert e <= Integer.MAX_VALUE;
		return hppcSet.contains((int) e);
	}

	@Override
	public int size()
	{
		return hppcSet.size();
	}

	@Override
	public boolean isEmpty()
	{
		return hppcSet.isEmpty();
	}

	@Override
	public Iterator<LongCursor> iterator()
	{
		final Iterator<IntCursor> i = hppcSet.iterator();

		return new Iterator<LongCursor>()
		{
			final LongCursor lc = new LongCursor();

			@Override
			public boolean hasNext()
			{
				return i.hasNext();
			}

			@Override
			public LongCursor next()
			{
				IntCursor ic = i.next();
				lc.value = ic.value;
				return lc;
			}

			@Override
			public void remove()
			{
				throw new IllegalStateException();
			}
		};
	}

	public float getLoadFactor()
	{
		return hppcSet.size() / (float) hppcSet.keys.length;
	}

	public int[] toArray()
	{
		int[] r = new int[size()];
		int i = 0;

		for (LongCursor c : this)
		{
			r[i++] = (int) c.value;

		}

		return r;
	}

	@Override
	public long getMemoryFootprintInBytes()
	{
		return hppcSet.keys.length * 4 + hppcSet.allocated.length * 1 + 8;
	}

}
