/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 /////////////////////////////////////////////////////////////////////////////////////////
// 
//                 Université de Nice Sophia-Antipolis  (UNS) - 
//                 Centre National de la Recherche Scientifique (CNRS)
//                 Copyright © 2015 UNS, CNRS All Rights Reserved.
// 
//     These computer program listings and specifications, herein, are
//     the property of Université de Nice Sophia-Antipolis and CNRS
//     shall not be reproduced or copied or used in whole or in part as
//     the basis for manufacture or sale of items without written permission.
//     For a license agreement, please contact:
//     <mailto: licensing@sattse.com> 
//
//
//
//     Author: Luc Hogie – Laboratoire I3S - luc.hogie@unice.fr
//
//////////////////////////////////////////////////////////////////////////////////////////

package dht.set;

import java.io.Externalizable;
import java.util.Iterator;
import java.util.Random;

import toools.Clazz;
import toools.MemoryObject;
import toools.collection.bigstuff.longset.LongCursor;

public abstract class LongSet
		implements Iterable<LongCursor>, Externalizable, MemoryObject
{
	public abstract Class<?> getImplementationClass();

	public abstract void add(long e);

	public abstract boolean removeFirstOccurence(long n);

	public abstract int removeAllOccurences(long n);

	public abstract boolean contains(long n);

	public abstract int size();

	public abstract long pickRandomElement(Random prng);

	@Override
	public abstract Iterator<LongCursor> iterator();

	public abstract void clear();

	public boolean isEmpty()
	{
		return size() == 0;
	}

	public void addAll(LongSet s)
	{
		for (LongCursor c : s)
		{
			add(c.value);
		}
	}

	public void addAll(long... a)
	{
		for (long n : a)
		{
			add(n);
		}
	}

	public void addAll(int... ids)
	{
		for (int id : ids)
		{
			add(id);
		}
	}

	public void removeFirstOccurences(LongSet s)
	{
		for (LongCursor c : s)
		{
			removeFirstOccurence(c.value);
		}
	}

	public void removeAllOccurences(LongSet s)
	{
		for (LongCursor c : s)
		{
			removeAllOccurences(c.value);
		}
	}

	public void retainAll(LongSet s)
	{
		for (LongCursor c : this)
		{
			if ( ! s.contains(c.value))
			{
				removeAllOccurences(c.value);
			}
		}
	}

	@Override
	public boolean equals(Object o)
	{
		LongSet otherSet = (LongSet) o;

		if (otherSet.size() == size())
		{
			return contains(otherSet);
		}
		else
		{
			return false;
		}
	}

	public boolean contains(LongSet otherSet)
	{
		// a set can't contain a set larger than itself
		if (otherSet.size() > size())
		{
			return false;
		}
		else
		{
			for (LongCursor c : otherSet)
			{
				if ( ! contains(c.value))
				{
					return false;
				}
			}

			return true;
		}
	}

	@Override
	public String toString()
	{
		StringBuilder b = new StringBuilder();
		Iterator<LongCursor> i = iterator();
		b.append('{');

		while (i.hasNext())
		{
			b.append(i.next().value);

			if (i.hasNext())
			{
				b.append(", ");
			}
		}

		b.append('}');
		return b.toString();
	}

	public String toString_numbers_only()
	{
		StringBuilder b = new StringBuilder();
		Iterator<LongCursor> i = iterator();

		while (i.hasNext())
		{
			b.append(i.next().value);

			if (i.hasNext())
			{
				b.append(' ');
			}
		}

		return b.toString();
	}

	public LongSet pickRandomSubset(Random prng, int numberOfElements, boolean remove)
	{
		if (numberOfElements > size())
			throw new IllegalArgumentException("cannot pick that many elements");

		LongSet r = null;

		if (numberOfElements == size())
		{
			r = clone();

			if (remove)
			{
				clear();
			}
		}
		else
		{
			r = Clazz.makeInstance(getClass());

			while (numberOfElements > r.size())
			{
				r.add(pickRandomElement(prng, r, remove));
			}
		}

		return r;
	}

	public long pickRandomElement(Random prng, int not, boolean remove)
	{
		while (true)
		{
			long e = pickRandomElement(prng);

			if (e != not)
			{
				if (remove)
				{
					removeAllOccurences(e);
				}

				return e;
			}
		}
	}

	public long pickRandomElement(Random prng, LongSet excludedElements, boolean remove)
	{
		while (true)
		{
			long e = pickRandomElement(prng);

			if ( ! excludedElements.contains(e))
			{
				if (remove)
				{
					removeAllOccurences(e);
				}

				return e;
			}
		}
	}

	@Override
	public LongSet clone()
	{
		return clone(getClass());
	}

	public LongSet clone(Class<? extends LongSet> c)
	{
		LongSet s = Clazz.makeInstance(c);
		s.addAll(this);
		return s;
	}

	public abstract boolean is64bit();
}
