/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 /////////////////////////////////////////////////////////////////////////////////////////
// 
//                 Université de Nice Sophia-Antipolis  (UNS) - 
//                 Centre National de la Recherche Scientifique (CNRS)
//                 Copyright © 2015 UNS, CNRS All Rights Reserved.
// 
//     These computer program listings and specifications, herein, are
//     the property of Université de Nice Sophia-Antipolis and CNRS
//     shall not be reproduced or copied or used in whole or in part as
//     the basis for manufacture or sale of items without written permission.
//     For a license agreement, please contact:
//     <mailto: licensing@sattse.com> 
//
//
//
//     Author: Luc Hogie – Laboratoire I3S - luc.hogie@unice.fr
//
//////////////////////////////////////////////////////////////////////////////////////////


package dht.set;

import java.util.Collection;
import java.util.Random;

import toools.Clazz;
import toools.collection.bigstuff.longset.LongCursor;


public class LongSets
{
	public static final LongSet emptySet = new ArrayLongSet64(new long[0]);

	public static <S extends LongSet> S union(Class<S> target, Collection<LongSet> sets)
	{
		S c = Clazz.makeInstance(target);

		for (LongSet set : sets)
		{
			c.addAll(set);
		}

		return c;
	}

	public static LongSet union(Collection<LongSet> sets)
	{
		return union(HashLongSet64.class, sets);
	}

	public static <S extends LongSet> S createRandomSet(Class<S> target, Random r, int n, int upperbound)
	{
		if (upperbound < n)
			throw new IllegalArgumentException("not enough elements available to create such a large set");

		S s = Clazz.makeInstance(target);

		while (s.size() < n)
		{
			s.add(r.nextInt(upperbound));
		}

		return s;
	}

	public static LongSet singleton(long n)
	{
		return new ArrayLongSet64(new long[] { n });
	}

	public static LongSet from(long... array)
	{
		return from(HashLongSet64.class, array);
	}

	public static <S extends LongSet> S from(Class<S> c, long... array)
	{
		S s = Clazz.makeInstance(c);
		s.addAll(array);
		return s;
	}

	public static <S extends LongSet> S intersection(Class<S> target, LongSet a, LongSet b)
	{
		LongSet smallestSet = a.size() < b.size() ? a : b;
		LongSet greatestSet = smallestSet == a ? b : a;

		S r = Clazz.makeInstance(target);

		for (LongCursor c : smallestSet)
		{
			if (greatestSet.contains(c.value))
			{
				r.add(c.value);
			}
		}

		return r;
	}

	public static LongSet intersection(LongSet a, LongSet b)
	{
		return intersection(HashLongSet64.class, a, b);
	}

	/**
	 * @param a
	 * @param b
	 * @return an IntSet that contains all elements of a that do not appear in
	 *         set b
	 */
	public static <S extends LongSet> S difference(Class<S> target, LongSet a, LongSet b)
	{
		S diff = Clazz.makeInstance(target);

		for (LongCursor i : a)
			if (!b.contains(i.value))
				diff.add(i.value);

		return diff;
	}

	public static LongSet difference(LongSet a, LongSet b)
	{
		return difference(HashLongSet64.class, a, b);
	}

}
