/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 package grph;

import grph.Grph.DIRECTION;
import toools.set.IntSet;

public interface GrphPrimitives
{
	DIRECTION getNavigation();

	// vertices
	IntSet getVertices();

	int getNextVertexAvailable();

	void removeVertex(int v);

	void addVertex(int v);
	boolean containsVertex(int v);

	IntSet getOutOnlyElements(int v);

	IntSet getInOnlyElements(int v);

	IntSet getInOutOnlyElements(int v);

	// edges
	boolean storeEdges();
	boolean containsEdge(int e);

	int getNextEdgeAvailable();

	IntSet getEdges();

	void removeEdge(int e);

	// undirected simple edges

	int getNumberOfUndirectedSimpleEdges();

	void addUndirectedSimpleEdge(int e, int v1, int v2);

	boolean isUndirectedSimpleEdge(int e);

	int getOneVertex(int e);

	int getTheOtherVertex(int e, int v);

	// directed simple edge
	int getNumberOfDirectedSimpleEdges();

	void addDirectedSimpleEdge(int src, int e, int dest);

	boolean isDirectedSimpleEdge(int e);

	int getDirectedSimpleEdgeTail(int e);

	int getDirectedSimpleEdgeHead(int e);

	// undirected hyper edges
	int getNumberOfUndirectedHyperEdges();

	void addUndirectedHyperEdge(int e);

	boolean isUndirectedHyperEdge(int e);

	void addToUndirectedHyperEdge(int edge, int vertex);

	void removeFromHyperEdge(int e, int v);

	IntSet getUndirectedHyperEdgeVertices(int e);

	// directed hyper edges
	void addDirectedHyperEdge(int e);

	boolean isDirectedHyperEdge(int e);

	int getNumberOfDirectedHyperEdges();

	IntSet getDirectedHyperEdgeHead(int e);

	IntSet getDirectedHyperEdgeTail(int e);

	void addToDirectedHyperEdgeTail(int e, int v);

	void addToDirectedHyperEdgeHead(int e, int v);

	void removeFromDirectedHyperEdgeHead(int e, int v);

	void removeFromDirectedHyperEdgeTail(int e, int v);

	void removeEdge(int u, int v);
}
