package java4unix;

import java.io.Console;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.Scanner;

import toools.ClassContainer;
import toools.Clazz;
import toools.io.file.AbstractFile;
import toools.io.file.Directory;
import toools.io.file.RegularFile;

public class ScriptJar
{
	private final AbstractFile f;

	public ScriptJar(AbstractFile jarfile)
	{
		this.f = jarfile;
	}

	public String guessApplicationName()
	{
		return f.getName().replaceAll("-.*", "");
	}

	public static Directory getInstallDir(String applicationName)
	{
		try
		{
			Directory dir = new Directory("/opt/" + applicationName);

			if (dir.exists())
			{
				RegularFile f = new RegularFile(dir, "fubar");
				f.create();
				f.delete();
			}
			else
			{
				dir.mkdirs();
			}

			return dir;
		}
		catch (Exception e)
		{
			// cannot install in /opt
			return new Directory(
					Directory.getHomeDirectory().getPath() + "/." + applicationName);
		}
	}

	public void deployLocally(boolean verbose) throws IOException
	{
		if (verbose)
			System.out.println("Scanning " + f);

		// strip anything that after a "-" in order to extract the
		// application name
		String expectedApplicationName = guessApplicationName();

		// ask the user if he wants something else
		System.out.println(
				"Application name (optional, found '" + expectedApplicationName + "'): ");
		String appNameGivenByTheUser = new Scanner(System.in).nextLine();

		if (appNameGivenByTheUser != null && appNameGivenByTheUser.length() > 0)
		{
			expectedApplicationName = appNameGivenByTheUser;
		}

		// search for the classes that correspond to scripts
		System.out
				.println("searching scripts for application " + expectedApplicationName);
		Collection<Class<AbstractShellScript>> scriptClasses = findScriptClasses(
				new ClassContainer(f), ".*");

		// if some classes were found
		if (scriptClasses.size() > 0)
		{
			for (Class<? extends AbstractShellScript> thisClass : scriptClasses)
			{
				AbstractShellScript script = Clazz.makeInstance(thisClass);

				if (script == null)
				{
					System.err.println(
							"Cannot instantiate script class " + thisClass.getName());
				}
				else
				{
					if (script.getApplicationName().equals(expectedApplicationName))
					{
						Directory installDir = getInstallDir(script.getApplicationName());

						if ( ! installDir.exists())
							installDir.mkdirs();

						script.setInstallationDirectory(installDir);

						if (verbose)
							System.out.println(
									"creating " + script.getExecutableScript().getPath());

						script.installScript();

						/*
						 * if (System.getProperty("user.name").equals("root")) {
						 * AbstractFile link = AbstractFile.map( "/opt/" +
						 * script.getExecutableScript().getName(),
						 * RegularFile.class); link.delete();
						 * Posix.makeSymbolicLink(script.getExecutableScript(),
						 * link); }
						 */
					}
					else
					{
						System.err.println("Script " + script.getClass().getName()
								+ " belongs to application " + script.getApplicationName()
								+ " instead of " + expectedApplicationName
								+ ". Skipping it.");
					}
				}
			}

			Directory binDir = getInstallDir(expectedApplicationName).getChild("bin",
					Directory.class);
			// setScript(binDir, expectedApplicationName + "-update",
			// "curl -s
			// http://www-sop.inria.fr/members/Luc.Hogie/java4unix/j4uni |
			// sh -s "
			// + expectedApplicationName,
			// verbose);
			// setScript(binDir, expectedApplicationName + "-uninstall",
			// "rm -rf " + binDir.getParent(), verbose);
		}
		else
		{
			System.out.println("No script class was found in class container: " + f);
		}
	}

	public static Collection<Class<AbstractShellScript>> findScriptClasses(
			ClassContainer cc, String re)
	{
		Collection<Class<AbstractShellScript>> scriptClasses = new HashSet<Class<AbstractShellScript>>();

		for (Class<?> thisClass : cc.listAllClasses(re))
		{
			if ( ! Clazz.isAbstract(thisClass)
					&& AbstractShellScript.class.isAssignableFrom(thisClass))
			{
				scriptClasses.add((Class<AbstractShellScript>) thisClass);
			}
		}

		return scriptClasses;
	}

	public static void main(String... args) throws IOException
	{
		// this is to force windows jdk to adopt Cygwin's home
		System.setProperty("user.home", System.getenv("HOME"));

		for (String name : args)
		{
			AbstractFile f = AbstractFile.map(name, null);
			ScriptJar sj = new ScriptJar(f);
			sj.deployLocally(true);
		}
	}

}
