/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 package toools.collection.bigstuff.large;

import toools.collections.LongIterator;
import toools.util.Conversion;

public abstract class AbstractLargeLongMap implements LargeLongMap
{
	protected final int slotCapacity;
	private final int nbSlots;

	public AbstractLargeLongMap(long capacity)
	{
		// don't waste computational resources
		this.nbSlots = Math.max(((int) (capacity / Integer.MAX_VALUE)) + 1,
				Runtime.getRuntime().availableProcessors());

		this.slotCapacity = Conversion.long2int(capacity / nbSlots);
	}

	protected int getSlotNumber(long key)
	{
		return Conversion.long2int(key / nbSlots);
	}

	@Override
	public long greatestKey()
	{
		LongIterator keyIterator = iterator();
		long max = Long.MIN_VALUE;

		while (keyIterator.hasNext())
		{
			long k = keyIterator.next();

			if (k > max)
			{
				max = k;
			}
		}

		return max;
	}

	public String toString(LongIterator longIterator)
	{
		StringBuilder b = new StringBuilder();

		while (longIterator.hasNext())
		{
			long v = longIterator.next();
			b.append(v + " => " + getStringOf(v) + "\n");
		}

		return b.toString();
	}

	protected abstract String getStringOf(long key);

	@Override
	public String toString()
	{
		StringBuilder b = new StringBuilder();
		b.append(getClass().getName() + " (" + size() + " entries) {");
		LongIterator i = iterator();

		while (i.hasNext())
		{
			long key = i.next();
			b.append("\n" + key + " => " + getStringOf(key));
		}

		b.append("\n}");
		return b.toString();
	}
	
}
