/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 
 package toools.collection.bigstuff.large;

import toools.collections.LongIterator;

public class LargeLongArray
{
	private long[][] a;

	public LargeLongArray(long sz)
	{
		a = new long[1 + (int) (sz / Integer.MAX_VALUE)][(int) (sz % Integer.MAX_VALUE)];
	}

	public long size()
	{
		// adds size of all segments except last one which may be smaller
		long size = (a.length - 1) * a[0].length;

		// adds the size of the last segment
		size += a[a.length - 1].length;
		return size;
	}

	public long get(long i)
	{
		return a[(int) (i / Integer.MAX_VALUE)][(int) (i % Integer.MAX_VALUE)];
	}

	public void set(long i, long v)
	{
		a[(int) (i / Integer.MAX_VALUE)][(int) (i % Integer.MAX_VALUE)] = v;
	}

	public LongIterator iterator()
	{
		return new LongIterator()
		{
			int i, j = 0;

			@Override
			public long next()
			{
				long v = a[i][j];

				if (++j >= a[i].length)
				{
					j = 0;
					++i;
				}

				return v;
			}

			@Override
			public boolean hasNext()
			{
				return i < a.length && j < a[a.length - 1].length;
			}
		};
	}

	public String toString()
	{
		StringBuilder s = new StringBuilder();
		s.append('[');

		LongIterator i = iterator();

		while (i.hasNext())
		{
			s.append(i.next());

			if (i.hasNext())
				s.append(' ');
		}

		s.append(']');
		return s.toString();
	}

	public static void main(String[] args)
	{
		LargeLongArray a = new LargeLongArray(5);
		System.out.println("size= " + a.size());
		a.set(2, 3);
		a.set(0, 1);

		LongIterator i = a.iterator();

		while (i.hasNext())
		{
			System.out.println(i.next());
		}
	}
}
