/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 


package toools.collection.bigstuff.longset;

import java.util.Iterator;
import java.util.Random;


import toools.math.MathsUtilities;

public class ArrayLongSet extends LongSet
{
	final long[] array;

	public ArrayLongSet(long[] array)
	{
		this.array = array;
	}

	@Override
	public Class<?> getImplementationClass()
	{
		return long[].class;
	}

	@Override
	public long[] toLongArray()
	{
		return array;
	}

	@Override
	public void add(final long n)
	{
		throw new IllegalStateException();
	}

	@Override
	public void remove(long n)
	{
		throw new IllegalStateException();
	}

	@Override
	public long pickRandomElement(Random prng)
	{
		return array[prng.nextInt(array.length)];
	}

	@Override
	public long getGreatest()
	{
		return MathsUtilities.max(array);
	}

	@Override
	public void clear()
	{
		throw new IllegalStateException();
	}

	@Override
	public boolean contains(long n)
	{
		if (array == null)
		{
			return false;
		}
		else
		{
			return indexOf(n) >= 0;
		}
	}

	public long indexOf(long n)
	{
		for (int i = 0; i < array.length; ++i)
		{
			if (array[i] == n)
			{
				return i;
			}
		}

		return - 1;
	}

	@Override
	public long size()
	{
		return array == null ? 0 : array.length;
	}

	@Override
	public boolean isEmpty()
	{
		return array == null || array.length == 0;
	}

	@Override
	public Iterator<LongCursor> iterator()
	{
		return new Iterator<LongCursor>()
		{
			int i = 0;
			final LongCursor c = new LongCursor();

			@Override
			public boolean hasNext()
			{
				return array != null && i < array.length;
			}

			@Override
			public LongCursor next()
			{
				c.value = array[i];
				return c;
			}

			@Override
			public void remove()
			{
				throw new IllegalStateException();
			}
		};
	}
}
