/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 


package toools.collection.bigstuff.longset;

import java.util.Iterator;
import java.util.Random;

import it.unimi.dsi.fastutil.longs.LongArrayList;
import toools.math.MathsUtilities;

public class LongArrayListSet extends LongSet
{
	private final LongArrayList list = new LongArrayList();
	private long max = - 1;

	@Override
	public Class<?> getImplementationClass()
	{
		return LongArrayList.class;
	}

	@Override
	public void add(long e)
	{
		if (e > max)
		{
			max = e;
		}

		list.add(e);
		fireAdd(e);
	}

	@Override
	public void remove(long e)
	{
		int index = list.indexOf(e);

		if (index < 0)
		{
			throw new IllegalArgumentException("this set doesn't contains element " + e);
		}
		else
		{
			list.remove(index);

			if (e == max)
			{
				max = isEmpty() ? - 1 : LongSets.findMax(this);
			}

			fireRemove(e);
		}
	}

	@Override
	public long pickRandomElement(Random prng)
	{
		return list.get(MathsUtilities.pickRandomBetween(0, list.size(), prng));
	}

	@Override
	public long getGreatest()
	{
		return max;
	}

	@Override
	public void clear()
	{
		list.clear();
	}

	@Override
	public boolean contains(long id)
	{
		return list.contains(id);
	}

	@Override
	public long size()
	{
		return list.size();
	}

	@Override
	public boolean isEmpty()
	{
		return list.isEmpty();
	}

	@Override
	public Iterator<LongCursor> iterator()
	{
		return LongCursor.fromFastUtil(list).iterator();
	}

}
