/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 


package toools.collection.bigstuff.longset;

import java.util.Iterator;
import java.util.Random;


public class SortedArrayLongSet extends LongSet
{
	private long[] array = new long[0];
	private boolean exposeInternalArray;

	public SortedArrayLongSet()
	{
		this(true);
	}

	public SortedArrayLongSet(boolean b)
	{
		exposeInternalArray = b;
	}

	@Override
	public Class<?> getImplementationClass()
	{
		return int[].class;
	}

	// Returns the position where the element n should be.
	// If n is already n, its index is returned.
	private final long indexOf(long n)
	{
		if (array.length == 0 || n <= array[0])
		{
			return 0;
		}
		else if (n == array[array.length - 1])
		{
			return array.length - 1;
		}
		else if (n > array[array.length - 1])
		{
			return array.length;
		}
		else
		{
			long i = 0;
			long j = array.length - 1;

			while (true)
			{
				long m = (i + j) / 2;

				if (n < array[(int) m])
				{
					j = m;
				}
				else if (n > array[(int) m])
				{
					i = m;
				}
				else
				{
					return m;
				}
			}
		}
	}

	@Override
	public long[] toLongArray()
	{
		if (exposeInternalArray)
		{
			if (array == null)
			{
				return array = new long[0];
			}
			else
			{
				return array;
			}
		}
		else
		{
			return super.toLongArray();
		}
	}

	@Override
	public void add(final long n)
	{
		if (array.length == 0)
		{
			array = new long[] { n };
		}
		else
		{
			int i = (int) indexOf(n);

			// if n is already in
			if (i < array.length && array[i] == n)
			{
				//throw new IllegalStateException("element " + n + " already in " + this);
			}
			else
			{
				long[] newlist = new long[array.length + 1];
				System.arraycopy(array, 0, newlist, 0, i);
				newlist[i] = n;
				System.arraycopy(array, i, newlist, i + 1, array.length - i);
				array = newlist;
			}
		}

		fireAdd(n);
	}

	@Override
	public void remove(long n)
	{
		if (array.length == 0)
		{
			throw new IllegalArgumentException("set is empty, cannot remove element");
		}
		else
		{
			int i = (int)indexOf(n);
System.out.println(i);
			// if n is in
			if (i < array.length && array[i] == n)
			{
				if (array.length == 1)
				{
					array = null;
				}
				else
				{
					long[] newlist = new long[array.length - 1];
					System.arraycopy(array, 0, newlist, 0, i);
					System.arraycopy(array, i + 1, newlist, i, array.length - i - 1);
					array = newlist;
				}

				fireRemove(n);
			}
			else
			{
				throw new IllegalArgumentException("element not in set");
			}
		}
	}

	@Override
	public long pickRandomElement(Random prng)
	{
		return array[prng.nextInt(array.length)];
	}

	@Override
	public long getGreatest()
	{
		return array[array.length - 1];
	}

	@Override
	public void clear()
	{
		array = null;
	}

	@Override
	public boolean contains(long n)
	{
		if (array == null)
		{
			return false;
		}
		else
		{
			int i = (int) indexOf(n);
			return i < array.length && array[i] == n;
		}
	}

	@Override
	public long size()
	{
		return array == null ? 0 : array.length;
	}

	@Override
	public boolean isEmpty()
	{
		return array == null || array.length == 0;
	}

	@Override
	public Iterator<LongCursor> iterator()
	{
		return new Iterator<LongCursor>() {
			int i = 0;
			final LongCursor c = new LongCursor();

			@Override
			public boolean hasNext()
			{
				return array != null && i < array.length;
			}

			@Override
			public LongCursor next()
			{
				c.value = array[i++];
				return c;
			}

			@Override
			public void remove()
			{
				throw new IllegalStateException();
			}
		};
	}

	public static void main(String[] args)
	{
		LongSet s = new SortedArrayLongSet();

		System.out.println(s);
		s.add(5);
		s.add(3);
		s.add(12);
		s.add(0);
		System.out.println(s);
		s.remove(0);
		System.out.println(s);
		s.remove(12);
		System.out.println(s);
		s.remove(3);
		System.out.println(s);
		s.remove(5);
		System.out.println(s);
	}

}
