/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 


package toools.collection.bigstuff.longset;

import java.util.Iterator;
import java.util.Random;


public final class UnmodifiableLongSet extends LongSet
{
	private final LongSet s;

	public UnmodifiableLongSet(LongSet s)
	{
		if (s == null)
			throw new NullPointerException();

		this.s = s;
	}

	@Override
	public Class<?> getImplementationClass()
	{
		return s.getClass();
	}

	@Override
	public void add(long e)
	{
		throw new IllegalStateException("modifying this set is not allowed");
	}

	@Override
	public void remove(long e)
	{
		throw new IllegalStateException("modifying this set is not allowed");
	}

	@Override
	public Iterator<LongCursor> iterator()
	{
		return s.iterator();
	}

	@Override
	public boolean contains(long id)
	{
		assert id >= 0 : id;
		return s.contains(id);
	}

	@Override
	public long size()
	{
		return s.size();
	}

	@Override
	public boolean isEmpty()
	{
		return s.isEmpty();
	}

	@Override
	public long pickRandomElement(Random prng)
	{
		return s.pickRandomElement(prng);
	}

	@Override
	public long getGreatest()
	{
		return s.getGreatest();
	}

	@Override
	public void clear()
	{
		throw new IllegalStateException("modifying this set is not allowed");
	}

	@Override
	public LongSet clone(Class<? extends LongSet> c)
	{
		return s.clone();
	}
}
