/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 


package toools.math;

import it.unimi.dsi.fastutil.longs.Long2LongMap;
import it.unimi.dsi.fastutil.longs.Long2LongOpenHashMap;
import it.unimi.dsi.fastutil.longs.Long2ObjectMap;
import it.unimi.dsi.fastutil.longs.Long2ObjectOpenHashMap;

/**
 * A matrix made of primitive longs. Indices for lines and columns begin to 0.
 */
public class LongLong2LongMap
{
	private final Long2ObjectMap<Long2LongMap> lines = new Long2ObjectOpenHashMap<>();

	public void clear()
	{
		lines.clear();
	}

	public boolean contains(long i, long j)
	{
		return lines.containsKey(i) && lines.get(i).containsKey(j);
	}

	public long get(long i, long j)
	{
		Long2LongMap line = null;

		if (lines.containsKey(i))
		{
			line = lines.get(i);

			if (line.containsKey(j))
			{
				return line.get(j);
			}
			else
			{
				return 0;
			}
		}
		else
		{
			return 0;
		}
	}

	public void set(long i, long j, long v)
	{
		Long2LongMap line = lines.get(i);

		if (line == null)
		{
			line = new Long2LongOpenHashMap();
			lines.put(i, line);
		}

		line.put(j, v);
	}

	public void remove(long i, long j)
	{
		Long2LongMap line = lines.get(i);

		if (line != null)
		{
			line.remove(j);
		}
	}

	public static void main(String[] args)
	{
		LongLong2LongMap m = new LongLong2LongMap();
		System.out.println(m);
		m.set(1, 0, 50);
		System.out.println(m);
	}
}
