/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 


package toools.set;

import java.util.Iterator;
import java.util.Random;

import com.carrotsearch.hppc.cursors.IntCursor;

public class CachingSelfAdaptiveSet extends SelfAdaptiveIntSet
{
	private int[] cache;
	public static boolean useCache = true;

	public CachingSelfAdaptiveSet(int offset)
	{
		super(offset);
	}
	
	public int[] getCache()
	{
		return cache;
	}

	/**
	 * <b>WARNING: using iterator() on a set when the cache is available entails
	 * a severe loss of performance. You should consider iterating over the
	 * output of toIntArray()</b>
	 */
	@Override
	public Iterator<IntCursor> iterator()
	{
		return super.iterator();
	}

	@Override
	public int size()
	{
		return cache != null ? cache.length : super.size();
	}

	@Override
	public boolean isEmpty()
	{
		return cache != null ? cache.length == 0 : super.isEmpty();
	}

	@Override
	public int pickRandomElement(Random prng)
	{
		if (size() == 0)
			throw new IllegalStateException("no element in set");

		if (cache != null)
		{
			return cache[prng.nextInt(cache.length)];
		}
		else
		{
			return super.pickRandomElement(prng);
		}
	}

	@Override
	public int[] toIntArray()
	{
		if (useCache)
		{
			if (cache == null)
			{
				cache = super.toIntArray();
			}

			return cache;
		}
		else
		{
			cache = null;
			return super.toIntArray();
		}
	}

	@Override
	public int getGreatest()
	{
		return super.getGreatest();
	}

	@Override
	public void add(int id)
	{
		cache = null;
		super.add(id);
	}

	@Override
	public void remove(int id)
	{
		cache = null;
		super.remove(id);
	}

	@Override
	public void clear()
	{
		super.clear();
		cache = null;
	}
}
