/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 


package toools.set;

import java.util.Iterator;
import java.util.Random;

import toools.math.MathsUtilities;

import com.carrotsearch.hppc.IntArrayList;
import com.carrotsearch.hppc.IntOpenHashSet;
import com.carrotsearch.hppc.cursors.IntCursor;

public class IntArrayListSet extends IntSet
{
	private final IntArrayList list = new IntArrayList();
	private int max = -1;

	@Override
	public Class<?> getImplementationClass()
	{
		return IntArrayList.class;
	}

	@Override
	public void add(int e)
	{
		if (e > max)
		{
			max = e;
		}

		list.add(e);
		fireAdd(e);
	}

	@Override
	public void remove(int e)
	{
		int index = list.indexOf(e);

		if (index < 0)
		{
			throw new IllegalArgumentException("this set doesn't contains element " + e);
		}
		else
		{
			list.remove(index);

			if (e == max)
			{
				max = isEmpty() ? -1 : IntSets.findMax(this);
			}

			fireRemove(e);
		}
	}

	@Override
	public int pickRandomElement(Random prng)
	{
		return list.get(MathsUtilities.pickRandomBetween(0, list.size(), prng));
	}

	@Override
	public int getGreatest()
	{
		return max;
	}

	@Override
	public void clear()
	{
		list.clear();
	}

	@Override
	public boolean contains(int id)
	{
		return list.contains(id);
	}

	@Override
	public int size()
	{
		return list.size();
	}

	@Override
	public boolean isEmpty()
	{
		return list.isEmpty();
	}

	@Override
	public Iterator<IntCursor> iterator()
	{
		return list.iterator();
	}

	public static void main(String[] args)
	{
		IntOpenHashSet s = new IntOpenHashSet();
		for (int i = 0; i < 10000; ++i)
		{
			s.add(new Random().nextInt());
		}
		System.out.println(s.assigned);
		System.out.println(s.size());
		System.out.println(s.keys.length);
		System.out.println(s.allocated.length);

	}

}
