/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 


package toools.set;

import java.util.Iterator;
import java.util.Random;

import toools.math.MathsUtilities;

import com.carrotsearch.hppc.cursors.IntCursor;

public class IntArrayWrappingIntSet extends IntSet
{
	final int[] array;

	public IntArrayWrappingIntSet(int[] array)
	{
		this.array = array;
	}

	@Override
	public Class<?> getImplementationClass()
	{
		return int[].class;
	}

	@Override
	public int[] toIntArray()
	{
		return array;
	}

	@Override
	public void add(final int n)
	{
		throw new IllegalStateException();
	}

	@Override
	public void remove(int n)
	{
		throw new IllegalStateException();
	}

	@Override
	public int pickRandomElement(Random prng)
	{
		return array[prng.nextInt(array.length)];
	}

	@Override
	public int getGreatest()
	{
		return MathsUtilities.max(array);
	}

	@Override
	public void clear()
	{
		throw new IllegalStateException();
	}

	@Override
	public boolean contains(int n)
	{
		if (array == null)
		{
			return false;
		}
		else
		{
			return indexOf(n) >= 0;
		}
	}

	public int indexOf(long n)
	{
		for (int i = 0; i < array.length; ++i)
		{
			if (array[i] == n)
			{
				return i;
			}
		}

		return -1;
	}

	@Override
	public int size()
	{
		return array == null ? 0 : array.length;
	}

	@Override
	public boolean isEmpty()
	{
		return array == null || array.length == 0;
	}

	@Override
	public Iterator<IntCursor> iterator()
	{
		return new Iterator<IntCursor>()
		{
			int i = 0;
			final IntCursor c = new IntCursor();

			@Override
			public boolean hasNext()
			{
				return array != null && i < array.length;
			}

			@Override
			public IntCursor next()
			{
				c.value = array[i];
				c.index = i++;
				return c;
			}

			@Override
			public void remove()
			{
				throw new IllegalStateException();
			}
		};
	}
}
