/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 


package toools.set;

import java.util.Iterator;
import java.util.Random;

import toools.math.MathsUtilities;

import com.carrotsearch.hppc.IntOpenHashSet;
import com.carrotsearch.hppc.cursors.IntCursor;

public class IntHashSet extends IntSet
{
	private IntOpenHashSet hppcSet = new IntOpenHashSet();
	private int max = -1;

	

	
	@Override
	public Class<?> getImplementationClass()
	{
		return IntOpenHashSet.class;
	}

	@Override
	public void add(int e)
	{

		if (e > max)
		{
			max = e;
		}

		hppcSet.add(e);
		fireAdd(e);
	}

	@Override
	public void remove(int e)
	{
		if (hppcSet.remove(e))
		{
			if (e == max)
			{
				max = hppcSet.isEmpty() ? -1 : IntSets.findMax(this);
			}

			fireRemove(e);
		}
		else
		{
			throw new IllegalArgumentException("this set doesn't contains element " + e);
		}
	}

	@Override
	public int pickRandomElement(Random prng)
	{
		while (true)
		{
			int i = MathsUtilities.pickRandomBetween(0, hppcSet.keys.length, prng);
			int n = hppcSet.keys[i];

			// if (set.contains(n))
			if (hppcSet.allocated[i])
			{
				return n;
			}
		}
	}

	@Override
	public int getGreatest()
	{
		return max;
	}

	@Override
	public void clear()
	{
		hppcSet.clear();
	}

	@Override
	public boolean contains(int id)
	{
		return hppcSet.contains(id);
	}

	@Override
	public int size()
	{
		return hppcSet.size();
	}

	@Override
	public boolean isEmpty()
	{
		return hppcSet.isEmpty();
	}

	@Override
	public Iterator<IntCursor> iterator()
	{
		return hppcSet.iterator();
	}

	public float getLoadFactor()
	{
		return hppcSet.size() / (float) hppcSet.keys.length;
	}

	public String toBitString()
	{
		StringBuilder b = new StringBuilder();

		for (int i = 0; i < hppcSet.keys.length; ++i)
		{
			b.append(' ');

			if (hppcSet.allocated[i])
			{
				b.append(hppcSet.keys[i]);
			}
			else
			{
				b.append('-');
			}

		}

		return b.toString();
	}

	public void ensureCapacity(int n)
	{
		IntOpenHashSet r = new IntOpenHashSet(n);
		r.addAll(hppcSet);
		this.hppcSet = r;
	}
}
