/* (C) Copyright 2009-2013 CNRS (Centre National de la Recherche Scientifique).

Licensed to the CNRS under one
or more contributor license agreements.  See the NOTICE file
distributed with this work for additional information
regarding copyright ownership.  The CNRS licenses this file
to you under the Apache License, Version 2.0 (the
"License"); you may not use this file except in compliance
with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on an
"AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, either express or implied.  See the License for the
specific language governing permissions and limitations
under the License.

*/

/* Contributors:

Luc Hogie (CNRS, I3S laboratory, University of Nice-Sophia Antipolis) 
Aurelien Lancin (Coati research team, Inria)
Christian Glacet (LaBRi, Bordeaux)
David Coudert (Coati research team, Inria)
Fabien Crequis (Coati research team, Inria)
Grégory Morel (Coati research team, Inria)
Issam Tahiri (Coati research team, Inria)
Julien Fighiera (Aoste research team, Inria)
Laurent Viennot (Gang research-team, Inria)
Michel Syska (I3S, Université Cote D'Azur)
Nathann Cohen (LRI, Saclay) 
Julien Deantoin (I3S, Université Cote D'Azur, Saclay) 

*/
 


package toools.set;

import java.util.Iterator;
import java.util.Random;

import com.carrotsearch.hppc.cursors.IntCursor;

public class IntSingletonSet extends IntSet
{
	private int element;

	@Override
	public Class<?> getImplementationClass()
	{
		return int.class;
	}

	public IntSingletonSet(int n)
	{
		this.element = n;
	}

	public void setValue(int n)
	{
		this.element = n;
	}

	@Override
	public void add(int e)
	{
		throw new IllegalStateException(
				"cannot add a new element to a singleton");
	}

	@Override
	public void remove(int e)
	{
		throw new IllegalStateException("cannot remove element from singleton");
	}

	@Override
	public int pickRandomElement(Random prng)
	{
		return element;
	}

	@Override
	public int getGreatest()
	{
		return element;
	}

	@Override
	public void clear()
	{
		throw new IllegalStateException("cannot remove element from singleton");
	}

	@Override
	public boolean contains(int id)
	{
		return element == id;
	}

	@Override
	public int size()
	{
		return 1;
	}

	@Override
	public boolean isEmpty()
	{
		return false;
	}

	@Override
	public Iterator<IntCursor> iterator()
	{
		return new Iterator<IntCursor>() {
			boolean seen = false;

			@Override
			public boolean hasNext()
			{
				return !seen;
			}

			@Override
			public IntCursor next()
			{
				if (seen)
				{
					throw new IllegalStateException("already seen");
				}
				else
				{
					IntCursor c = new IntCursor();
					c.value = element;
					seen = true;
					return c;
				}
			}

			@Override
			public void remove()
			{
				throw new IllegalStateException(
						"cannot remove element from singleton");
			}
		};
	}

	public static void main(String[] args)
	{
		// IDSet s = new ListSet();
		IntSet s = new IntSingletonSet(5);
		System.out.println(s.size());
		s.add(9);
		System.out.println(s.size());
		System.out.println(s);

	}

}
